import React from 'react'
import { Query, compose, graphql } from 'react-apollo'
import { H4 } from '@pubsweet/ui'
import { pageSize } from 'config'
import { Loading, LoadingIcon, Notification } from '../ui'
import { ALL_MANUSCRIPTS, ADMIN_MANUSCRIPTS, ADD_REVIEWER } from './operations'
import DashboardBase from './DashboardBase'
import DashboardList from './DashboardList'
import SearchBoxes from './SearchBoxes'

const MyQueue = ({ currentUser, errors, setErrors }) => (
  <Query
    fetchPolicy="cache-and-network"
    query={ADMIN_MANUSCRIPTS}
    variables={{ external: currentUser.external }}
  >
    {({ data, loading }) => {
      if (loading) {
        return (
          <Loading>
            <LoadingIcon />
          </Loading>
        )
      }
      const mine = []
      const claimed = []
      const unclaimed = []
      data.adminManuscripts.forEach(m => {
        if (m.claiming) {
          if (m.claiming.id === currentUser.id) {
            mine.push(m)
          } else {
            claimed.push(m)
          }
        } else {
          unclaimed.push(m)
        }
      })
      return (
        <React.Fragment>
          {errors.map(e => (
            <Notification key="message" type="warning">
              {e.message}
            </Notification>
          ))}
          {mine.length > 0 && (
            <React.Fragment>
              <H4>My claimed manuscripts ({mine.length})</H4>
              <DashboardList
                currentUser={currentUser}
                errors={setErrors}
                listData={mine}
                sectionRole="admin"
              />
            </React.Fragment>
          )}
          {unclaimed.length > 0 && (
            <React.Fragment>
              <H4>Unclaimed ({unclaimed.length})</H4>
              <DashboardList
                currentUser={currentUser}
                errors={setErrors}
                listData={unclaimed}
                sectionRole="admin"
              />
            </React.Fragment>
          )}
          {claimed.length > 0 && (
            <React.Fragment>
              <H4>In process ({claimed.length})</H4>
              <DashboardList
                currentUser={currentUser}
                errors={setErrors}
                listData={claimed}
                sectionRole="admin"
              />
            </React.Fragment>
          )}
        </React.Fragment>
      )
    }}
  </Query>
)

const MyManuscripts = ({ currentUser, errors }) => (
  <Query fetchPolicy="cache-and-network" query={ALL_MANUSCRIPTS}>
    {({ data, loading }) => {
      if (loading) {
        return (
          <Loading>
            <LoadingIcon />
          </Loading>
        )
      }
      const attention = []
      const review = []
      const complete = []
      const manuscripts = []
      data.manuscripts.forEach(m => {
        const reviewer = m.teams.find(t => t.role === 'reviewer')
        const submitter = m.teams.find(t => t.role === 'submitter')
        if (
          reviewer &&
          reviewer.teamMembers[0].user.id === currentUser.id &&
          ['in-review', 'xml-review'].includes(m.status)
        ) {
          review.push(m)
        } else if (
          submitter.teamMembers[0].user.id === currentUser.id &&
          ['INITIAL', 'READY', 'submission-error'].includes(m.status)
        ) {
          attention.push(m)
        } else if (
          ['xml-complete', 'ncbi-ready', 'published'].includes(m.status)
        ) {
          complete.push(m)
        } else {
          manuscripts.push(m)
        }
      })
      return (
        <React.Fragment>
          {data.manuscripts.length === 0 && (
            <Notification type="info">
              There are currently no manuscripts connected to this account.
            </Notification>
          )}
          {data.manuscripts.length > pageSize && <SearchBoxes />}
          {errors.map(e => (
            <Notification key="message" type={e.type ? e.type : 'warning'}>
              {e.message}
            </Notification>
          ))}
          {attention.length > 0 && (
            <React.Fragment>
              <H4>Needs attention ({attention.length})</H4>
              <DashboardList listData={attention} sectionRole="submitter" />
            </React.Fragment>
          )}
          {review.length > 0 && (
            <React.Fragment>
              <H4>Needs review ({review.length})</H4>
              <DashboardList listData={review} sectionRole="reviewer" />
            </React.Fragment>
          )}
          {manuscripts.length > 0 && (
            <React.Fragment>
              <H4>In process ({manuscripts.length})</H4>
              <DashboardList currentUser={currentUser} listData={manuscripts} />
            </React.Fragment>
          )}
          {complete.length > 0 && (
            <React.Fragment>
              <H4>Submission complete ({complete.length})</H4>
              <DashboardList listData={complete} sectionRole="submitter" />
            </React.Fragment>
          )}
        </React.Fragment>
      )
    }}
  </Query>
)

const QueuePage = DashboardBase(MyQueue)
const ManuscriptsPage = DashboardBase(MyManuscripts)

class DashboardPage extends React.Component {
  state = { errors: [] }
  componentDidMount() {
    const { search, pathname } = this.props.location
    if (search) {
      const noteId = search.split('=')[1]
      this.props.history.replace(pathname)
      this.setReviewer(noteId)
    }
  }
  componentDidUpdate() {
    if (this.state.errors.length > 0) {
      setTimeout(() => {
        this.setState({ errors: [] })
      }, 4000)
    }
  }
  async setReviewer(noteId) {
    const set = await this.props.addReviewer({
      variables: { noteId },
      refetchQueries: [{ query: ALL_MANUSCRIPTS }],
      awaitRefetchQueries: true,
    })
    if (!set.data.addReviewer) {
      this.setState({
        errors: [
          {
            message: 'Manuscript already accepted for review.',
            type: 'warning',
          },
        ],
      })
    } else {
      this.setState({
        errors: [
          {
            message: 'Manuscript accepted for review.',
            type: 'success',
          },
        ],
      })
    }
  }
  render() {
    const { currentUser } = this.props
    if (currentUser.admin || currentUser.external) {
      return (
        <QueuePage
          currentUser={currentUser}
          errors={this.state.errors}
          pageTitle="My queue"
          setErrors={errors => this.setState({ errors })}
        />
      )
    }
    return (
      <ManuscriptsPage
        currentUser={currentUser}
        errors={this.state.errors}
        pageTitle="My manuscripts"
      />
    )
  }
}

export default compose(graphql(ADD_REVIEWER, { name: 'addReviewer' }))(
  DashboardPage,
)
