import React from 'react'
import { withRouter } from 'react-router'
import { Mutation } from 'react-apollo'
import styled from 'styled-components'
import { th } from '@pubsweet/ui-toolkit'
import { H3, H4, Button, Icon, Link } from '@pubsweet/ui'
import { B, Buttons, Close, CloseButton, Portal, Notification } from '../ui'
import { States, timeSince } from '../dashboard'
import Citation from './Citation'
import { REPLACE_MANUSCRIPT, CHECK_DUPES } from './operations'

const Columns = styled.div`
  display: flex;
  flex-wrap: wrap;
  align-items: stretch;
  justify-content: flex-end;
  h3 {
    margin-top: 0;
  }
  h4 {
    margin: 0;
  }
  button {
    display: flex;
    align-items: center;
  }
  & > * {
    width: 48%;
    box-sizing: border-box;
    margin-right: 1%;
    margin-left: 1%;
  }
  & > div {
    margin-top: ${th('gridUnit')};
    display: flex;
    align-items: stretch;
    justify-content: space-between;
    flex-direction: column;
    border: ${th('borderWidth')} ${th('borderStyle')} ${th('colorBorder')};
    padding: calc(${th('gridUnit')} * 2);
  }
`
const Current = styled.div`
  background-color: ${th('colorBackground')};
`
const Dupes = styled.div`
  background-color: ${th('colorBackgroundHue')};
`
const FlexP = styled.div`
  display: flex;
  flex-wrap: wrap;
  align-items: center;
  justify-content: space-between;
`
class ResolveDuplicates extends React.Component {
  state = {
    error: false,
    duplicates: this.props.duplicates,
  }
  unsetDupe = () => {
    const { duplicates } = this.state
    const { current } = this.props
    if (
      current.meta.articleIds.find(id => id.pubIdType === 'pmid').id ===
      duplicates[0].meta.articleIds.find(id => id.pubIdType === 'pmid').id
    ) {
      this.setState({ error: true })
    } else {
      duplicates.shift()
      this.setState({ duplicates })
    }
  }
  render() {
    const { close, current } = this.props
    const { duplicates, error } = this.state
    return (
      <Mutation
        mutation={REPLACE_MANUSCRIPT}
        refetchQueries={result => {
          if (result.data.replaceManuscript) {
            return [
              {
                query: CHECK_DUPES,
                variables: {
                  id: current.id,
                  pmid:
                    current.meta.articleIds &&
                    current.meta.articleIds.find(id => id.pubIdType === 'pmid')
                      ? current.meta.articleIds.find(
                          id => id.pubIdType === 'pmid',
                        ).id
                      : null,
                  title: current.meta.title,
                },
              },
            ]
          }
          return []
        }}
      >
        {(replaceManuscript, { data }) => {
          const replace = async (keepId, throwId) => {
            const { data } = await replaceManuscript({
              variables: { keepId, throwId },
            })
            if (data.replaceManuscript && throwId === current.id) {
              this.props.history.push('/')
            }
          }
          return (
            <Portal transparent>
              <Close>
                <CloseButton onClick={close} />
              </Close>
              <Columns>
                <H3>This manuscript</H3>
                <H3>Duplicate(s)</H3>
                {error && (
                  <div style={{ flex: '0 0 100%' }}>
                    <Notification type="error">
                      Two articles cannot have the same PMID.
                    </Notification>
                  </div>
                )}
                <Current>
                  <H4>{current.id}</H4>
                  <Citation journal={current.journal} metadata={current.meta} />
                  <FlexP>
                    <span>
                      <B>Status:</B> {current.status}
                    </span>
                    <span>
                      <B>Last updated:</B> {timeSince(current.updated)}
                    </span>
                  </FlexP>
                  <p>
                    <B>Grants: </B>
                    {current.meta.fundingGroup &&
                      current.meta.fundingGroup.map((f, t) => (
                        <span key={f.awardId}>
                          {`${f.fundingSource} ${f.awardId}`}
                          {t !== current.meta.fundingGroup.length - 1 && ', '}
                        </span>
                      ))}
                  </p>
                  <Button
                    onClick={() => replace(current.id, duplicates[0].id)}
                    primary
                  >
                    Remove this &amp; transfer grants
                    <Icon color="currentColor">arrow-right</Icon>
                  </Button>
                </Current>
                {duplicates.map((dupe, i) => (
                  <React.Fragment key={dupe.id}>
                    <Dupes>
                      <H4>{dupe.id}</H4>
                      <Link
                        target="_blank"
                        to={`/submission/${dupe.id}/${
                          States.admin[dupe.status].url
                        }`}
                      >
                        <Citation journal={dupe.journal} metadata={dupe.meta} />
                      </Link>
                      <FlexP>
                        <span>
                          <B>Status:</B> {dupe.status}
                        </span>
                        <span>
                          <B>Last updated:</B> {timeSince(dupe.updated)}
                        </span>
                      </FlexP>
                      <p>
                        <B>Grants: </B>
                        {dupe.meta.fundingGroup &&
                          dupe.meta.fundingGroup.map((f, t) => (
                            <span key={f.awardId}>
                              {`${f.fundingSource} ${f.awardId}`}
                              {t !== dupe.meta.fundingGroup.length - 1 && ', '}
                            </span>
                          ))}
                      </p>
                      <Button
                        onClick={() => replace(current.id, dupe.id)}
                        primary
                      >
                        <Icon color="currentColor">arrow-left</Icon>
                        Transfer grants &amp; remove this
                      </Button>
                    </Dupes>
                    {/* i === 0 && (
                      <p style={{ flex: '0 0 100%', textAlign: 'center' }}>
                        <Button
                          onClick={() => this.unsetDupe()}
                          style={{ margin: '0 auto' }}
                        >
                          These two are not duplicates!
                        </Button>
                      </p>
                    ) */}
                    {i !== duplicates.length - 1 && <br />}
                  </React.Fragment>
                ))}
              </Columns>
              <Buttons>
                <Button onClick={close}>Exit</Button>
              </Buttons>
            </Portal>
          )
        }}
      </Mutation>
    )
  }
}

export default withRouter(ResolveDuplicates)
