const chokidar = require('chokidar')
const config = require('config')
const logger = require('@pubsweet/logger')
const getUser = require('../utils/user.js')
const tar = require('../utils/unTar.js')
const files = require('../utils/files.js')
const db = require('../utils/db.js')

const ftpTagger = config.get('ftp_tagger')
const rootPath = `${process.env.HOME}/${config.get('ftp_directory')}/${
  ftpTagger.username
}/Done`

const ignoreNewFolder = `${rootPath}/New`

// chokidar monitors tagged folder
const watcher = chokidar.watch(`${rootPath}**/*.tar.gz`, {
  persistent: true,
  ignored: ignoreNewFolder,
  awaitWriteFinish: {
    stabilityThreshold: 2000,
    pollInterval: 100,
  },
})

watcher
  .on('add', path => logger.debug(`File ${path} has been added.`))
  .on('add', path => processFile(path))
  .on('change', path => logger.debug(`File ${path} has been changed`))
  .on('unlink', path => logger.debug(`File ${path} has been removed`))
  .on('error', error => logger.error(`Watcher error: ${error}`))
  .on('ready', () =>
    logger.info(
      `Initial scan complete. Ready for changes under folder ${rootPath}`,
    ),
  )
  .on('raw', (event, path, details) => {
    logger.debug('Raw event info:', event, path, details)
  })

watcher.on('change', (path, stats) => {
  if (stats) logger.debug(`File ${path} changed size to ${stats.size}`)
})

async function processFile(path) {
  try {
    logger.info(`Processing tagged XML packages: ${path}`)

    const user = await getUser.getFTPUser(rootPath, path)
    const tmpPath = await tar.createTempDir()
    const extractedFilePath = await tar.untar(path, tmpPath)
    const manifestFilename = await files.getManifestFilename(extractedFilePath)
    const filesData = await files.getManifestFileData(
      extractedFilePath,
      manifestFilename,
    )
    const [filesArr, promisesArr] = await files.checkFiles(
      filesData,
      tmpPath,
      user,
    )
    await db.createTaggersFiles(filesArr)
    await Promise.all(promisesArr)
    await files.renameFile(path)
    logger.info('Uploading to Minio and the database has been completed.')
  } catch (err) {
    logger.error('Error', err.message)
  }
}
